/* ***** BEGIN LICENSE BLOCK *****
 * This file is part of Natron <http://www.natron.fr/>,
 * Copyright (C) 2013-2017 INRIA and Alexandre Gauthier-Foichat
 *
 * Natron is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Natron is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Natron.  If not, see <http://www.gnu.org/licenses/gpl-2.0.html>
 * ***** END LICENSE BLOCK ***** */

#ifndef NONKEYPARAMSSERIALIZATION_H
#define NONKEYPARAMSSERIALIZATION_H

// ***** BEGIN PYTHON BLOCK *****
// from <https://docs.python.org/3/c-api/intro.html#include-files>:
// "Since Python may define some pre-processor definitions which affect the standard headers on some systems, you must include Python.h before any standard headers are included."
#include <Python.h>
// ***** END PYTHON BLOCK *****

#include "Global/Macros.h"

#include "Engine/NonKeyParams.h"

#if !defined(Q_MOC_RUN) && !defined(SBK_RUN)
GCC_DIAG_UNUSED_LOCAL_TYPEDEFS_OFF
GCC_DIAG_OFF(unused-parameter)
// /opt/local/include/boost/serialization/smart_cast.hpp:254:25: warning: unused parameter 'u' [-Wunused-parameter]
#include <boost/archive/binary_iarchive.hpp>
#include <boost/archive/binary_oarchive.hpp>

//Should not be needed but doesn't work without explicit instanciation with XML
#include <boost/archive/xml_iarchive.hpp>
#include <boost/archive/xml_oarchive.hpp>
GCC_DIAG_UNUSED_LOCAL_TYPEDEFS_ON
GCC_DIAG_ON(unused-parameter)
#endif

#include "Engine/EngineFwd.h"

NATRON_NAMESPACE_ENTER

template<class Archive>
void
NonKeyParams::serialize(Archive & ar,
                        const unsigned int /*version*/)
{
    ar & ::boost::serialization::make_nvp("DataTypeSize", _storageInfo.dataTypeSize);
    ar & ::boost::serialization::make_nvp("NComps", _storageInfo.numComponents);
    ar & ::boost::serialization::make_nvp("Bounds", _storageInfo.bounds);
    ar & ::boost::serialization::make_nvp("StorageMode", _storageInfo.mode);
    ar & ::boost::serialization::make_nvp("TexTarget", _storageInfo.textureTarget);
}

NATRON_NAMESPACE_EXIT

BOOST_SERIALIZATION_ASSUME_ABSTRACT(NATRON_NAMESPACE::NonKeyParams);

#endif // NONKEYPARAMSSERIALIZATION_H
