/* ***** BEGIN LICENSE BLOCK *****
 * This file is part of Natron <http://www.natron.fr/>,
 * Copyright (C) 2013-2017 INRIA and Alexandre Gauthier-Foichat
 *
 * Natron is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Natron is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Natron.  If not, see <http://www.gnu.org/licenses/gpl-2.0.html>
 * ***** END LICENSE BLOCK ***** */

// ***** BEGIN PYTHON BLOCK *****
// from <https://docs.python.org/3/c-api/intro.html#include-files>:
// "Since Python may define some pre-processor definitions which affect the standard headers on some systems, you must include Python.h before any standard headers are included."
#include <Python.h>
// ***** END PYTHON BLOCK *****

#include "Button.h"

#include <stdexcept>

#include <QApplication>

#include "Gui/GuiApplicationManager.h"

NATRON_NAMESPACE_ENTER

Button::Button(QWidget* parent)
    : QPushButton(parent)
{
    setAttribute(Qt::WA_LayoutUsesWidgetRect); // Don't use the layout rect calculated from QMacStyle.
    initInternal();
}

Button::Button(const QString & text,
               QWidget * parent)
    : QPushButton(text, parent)
{
    setAttribute(Qt::WA_LayoutUsesWidgetRect); // Don't use the layout rect calculated from QMacStyle.
    initInternal();
}

Button::Button(const QIcon & icon,
               const QString & text,
               QWidget * parent)
    : QPushButton(icon, text, parent)
{
    setAttribute(Qt::WA_LayoutUsesWidgetRect); // Don't use the layout rect calculated from QMacStyle.
    initInternal();
}

void
Button::initInternal()
{
    setFont( QApplication::font() ); // necessary, or the buttons will get the default font size
    //setFont( QFont(appFont,appFontSize) );
}

NATRON_NAMESPACE_EXIT

