/* ***** BEGIN LICENSE BLOCK *****
 * This file is part of Natron <http://www.natron.fr/>,
 * Copyright (C) 2013-2017 INRIA and Alexandre Gauthier-Foichat
 *
 * Natron is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Natron is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Natron.  If not, see <http://www.gnu.org/licenses/gpl-2.0.html>
 * ***** END LICENSE BLOCK ***** */

// ***** BEGIN PYTHON BLOCK *****
// from <https://docs.python.org/3/c-api/intro.html#include-files>:
// "Since Python may define some pre-processor definitions which affect the standard headers on some systems, you must include Python.h before any standard headers are included."
#include <Python.h>
// ***** END PYTHON BLOCK *****

#include "RightClickableWidget.h"

#include <stdexcept>

#include <QtCore/QThread>
#include <QApplication>
GCC_DIAG_UNUSED_PRIVATE_FIELD_OFF
// /opt/local/include/QtGui/qmime.h:119:10: warning: private field 'type' is not used [-Wunused-private-field]
#include <QMouseEvent>
#include <QKeyEvent>
GCC_DIAG_UNUSED_PRIVATE_FIELD_ON
#include <QToolButton>

#include "Gui/CurveWidget.h"
#include "Gui/GuiMacros.h"
#include "Gui/Histogram.h"
#include "Gui/NodeGraph.h"
#include "Gui/ViewerGL.h"

NATRON_NAMESPACE_ENTER

void
RightClickableWidget::mousePressEvent(QMouseEvent* e)
{
    if ( buttonDownIsRight(e) ) {
        QWidget* underMouse = qApp->widgetAt( e->globalPos() );
        if (underMouse == this) {
            Q_EMIT rightClicked( e->pos() );
            e->accept();
        }
    } else {
        QWidget::mousePressEvent(e);
    }
}

void
RightClickableWidget::keyPressEvent(QKeyEvent* e)
{
    if (e->key() == Qt::Key_Escape) {
        Q_EMIT escapePressed();
    }
    QWidget::keyPressEvent(e);
}

void
RightClickableWidget::enterEvent(QEvent* e)
{
    // always running in the main thread
    assert( qApp && qApp->thread() == QThread::currentThread() );

    QWidget* currentFocus = qApp->focusWidget();
    bool canSetFocus = !currentFocus ||
                       dynamic_cast<ViewerGL*>(currentFocus) ||
                       dynamic_cast<CurveWidget*>(currentFocus) ||
                       dynamic_cast<Histogram*>(currentFocus) ||
                       dynamic_cast<NodeGraph*>(currentFocus) ||
                       dynamic_cast<QToolButton*>(currentFocus) ||
                       currentFocus->objectName() == QString::fromUtf8("Properties");

    if (canSetFocus) {
        setFocus();
    }
    QWidget::enterEvent(e);
}

RightClickableWidget*
RightClickableWidget::isParentSettingsPanelRecursive(QWidget* w)
{
    if (!w) {
        return 0;
    }
    RightClickableWidget* panel = qobject_cast<RightClickableWidget*>(w);
    if (panel) {
        return panel;
    } else {
        return isParentSettingsPanelRecursive( w->parentWidget() );
    }
}

NATRON_NAMESPACE_EXIT

NATRON_NAMESPACE_USING
#include "moc_RightClickableWidget.cpp"
